/// <reference types="node" />
/**
 * TextPacket class to create text packets such as for actions or other uses.
 */
declare class TextPacket {
    type: number;
    strings: string[];
    /**
     * Creates a new TextPacket class
     * @param type The type of the packet.
     * @param strings An array of strings to include, they will be joined by a newline character. (`\n`)
     */
    constructor(type: number, strings: string[]);
    /**
     * Creates a TextPacket class.
     * @param type The type of the packet.
     * @param strings Strings to include to the packet. They are not arrays, but instead they are arguments for the function.
     */
    static from(type: number, ...strings: string[]): TextPacket;
    /**
     * Creates a TextPacket class from a Buffer.
     * @param packet The buffer to convert.
     */
    static fromBuffer(packet: Buffer): TextPacket;
    /**
     * Parses the TextPacket and covnerts it to a buffer.
     */
    parse(): Buffer;
}
export { TextPacket };
